"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toGraphQLQueryString = toGraphQLQueryString;

var _lodash = require("lodash.isobject");

var _lodash2 = _interopRequireDefault(_lodash);

var _flow = require("./flow");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function objectToGraphQLString(obj) {
  var level = arguments.length <= 1 || arguments[1] === undefined ? 1 : arguments[1];

  var indentation = "  ".repeat(level);

  var strings = Object.keys(obj).reduce(function (parts, key) {
    var value = obj[key];
    var str = key;
    if ((0, _lodash2.default)(value)) {
      str = str + " {\n" + objectToGraphQLString(value, level + 1) + "\n" + indentation + "}";
    }
    parts.push(indentation + str);
    return parts;
  }, []);

  return strings.join(",\n");
}

var uppercaseFirstChar = function uppercaseFirstChar(str) {
  return str.charAt(0).toUpperCase() + str.slice(1);
};

function toGraphQLQueryString(objectType, relayContainerFragments, fragmentName) {
  var fragmentKey = objectType.key.name;
  var typeAnnotation = objectType.value;
  var obj = typeAnnotation.type === "ObjectTypeAnnotation" ? (0, _flow.convertFlowObjectTypeAnnotation)(typeAnnotation) : {};

  var graphQlQueryBody = objectToGraphQLString(obj);

  var childRelayContainersForFragment = Object.keys(relayContainerFragments).reduce(function (c, name) {
    var fragmentKeys = relayContainerFragments[name];
    if (fragmentKeys.indexOf(fragmentKey) >= 0) {
      c.push(name);
    }
    return c;
  }, []);

  var graphQlQuery = "\n" + "fragment on " + (fragmentName || uppercaseFirstChar(fragmentKey)) + " {\n" + graphQlQueryBody;
  var graphQlQueryEnd = "\n}\n";

  if (childRelayContainersForFragment.length > 0) {
    if (graphQlQueryBody) {
      graphQlQuery = graphQlQuery + ",\n  ";
    }

    graphQlQuery = graphQlQuery + childRelayContainersForFragment.map(function (name) {
      return "${" + name + ".getFragment('" + fragmentKey + "')}";
    }).join(",\n  ");
  }
  graphQlQuery = graphQlQuery + graphQlQueryEnd;

  // const buildRequire = template(`
  //   var IMPORT_NAME = require(SOURCE);
  // `);

  return "() => Relay.QL`" + graphQlQuery + "`";
}